# MT::Plugins::Redland::CMS::Topic::Image
# -----------------
# $Revision: 1.6 $
# $Date: 2005/04/11 05:39:23 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS::Topic::Image;

use strict;
use warnings;
no warnings 'once';

use Carp qw(confess);
use Encode;
use IO::All;
use File::Spec;
use Data::Dumper;
use RDF::Redland;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME			= "Redland Image Topic";
	$VERSION		= do { my @REV = split(/\./, (qw$Revision: 1.6 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug			= 0;
	$plugin_data	= {
		name		=> "${NAME}, v${VERSION}",
		description => 'Adds an image topic to an Entry',
		doc_link	=> 'http://kasei.us/code/mtredland/'
	};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms_topic.pl' );
	require $cms;
}

######################################################################

MT::Plugins::Redland::CMS->install_cms_header( ${NAME}, <<"END" );
<style type="text/css">
	#Image input { width: 450px; }
</style>
END
MT::Plugins::Redland::CMS::Topic->install_topic( 'Image', sub {
	my $app		= shift;
	my $q		= $app->{query};
	my $id		= $q->param('id');
	my $type	= $q->param('_type');
	my $class	= $app->_load_driver_for($type) or return;
	my ($url, $thumb, $title)	= ('') x 3;
	if ($id) {          # object exists, we're just editing it.
		my $entry	= $class->load($id) or return $app->error($app->translate("Load failed: [_1]", $class->errstr || "(no reason given)"));
		$url		= $entry->image_url;
		$thumb		= $entry->image_thumbnail_url;
		$title		= $entry->image_title;
	}
	return qq(<p><label for="image_url">URL</label>: <input name="image_url" id="image_url" value="${url}" /></p>)
		. qq(<p><label for="image_thumbnail_url">Thumbnail URL</label>: <input name="image_thumbnail_url" id="image_thumbnail_url" value="${thumb}" /></p>)
		. qq(<p><label for="image_title">Title</label>: <input name="image_title" id="image_title" value="${title}" /></p>);
});

no warnings 'redefine';
my $orig	= MT::Entry->can('column_names');
*MT::Entry::column_names	= sub {
	my $self	= shift;
	my $names	= $orig->( $self, @_ );
	push(@{ $names }, qw(image_url image_title image_thumbnail_url));
	return $names;
};

MT::Entry->add_callback("post_load", 4, $plugin_data, sub {
	my ($cb, $terms, $args, $entry) = @_;
	my $model	= $entry->driver->model;
	my $t_image	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/Image' );
	my $topic	= $entry->rdf_topic_of_type( $t_image );
	if ($topic) {
		my $p_thumb	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/thumbnail' );
		my $p_title	= RDF::Redland::Node->new_from_uri( 'http://purl.org/dc/elements/1.1/title' );
		my ($url)	= ($topic->is_resource) ? $topic->uri->as_string : '';
		if ($url) {
			$entry->image_url( $url );
		}
		my ($thumb)	= $model->targets( $topic, $p_thumb );
		if ($thumb and $thumb->is_resource) {
			$entry->image_thumbnail_url( $thumb->uri->as_string );
		}
		my ($title)	= $model->targets( $topic, $p_title );
		if ($title and $title->is_literal) {
			$entry->image_title( decode('utf8', $title->literal_value) );
		}
	}
});

MT::Entry->add_callback("pre_save", 4, $plugin_data, sub {
	my ($cb, $entry)	= @_;
	my $model	= $entry->driver->model;
	my $node	= $entry->get_rdf_node;
	my $t_image	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/Image' );
	my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
	my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	my $p_thumb	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/thumbnail' );
	my $p_title	= RDF::Redland::Node->new_from_uri( 'http://purl.org/dc/elements/1.1/title' );
	
	if (my $url = delete $entry->{'column_values'}{'image_url'}) {
		$entry->remove_rdf_topics_of_type( $t_image );
		my ($image)	= RDF::Redland::Node->new_from_uri( $url );
		my $tst		= RDF::Redland::Statement->new( $image, $p_type, $t_image );
		unless ($model->contains_statement( $tst )) {
			$model->add_statement( $tst );
		}
		my $st		= RDF::Redland::Statement->new( $node, $p_topic, $image );
		unless ($model->contains_statement( $st )) {
			$model->add_statement( $st );
		}
		if (my $turl = delete $entry->{'column_values'}{'image_thumbnail_url'}) {
			my $node	= $entry->get_rdf_node;
			my ($thumb)	= RDF::Redland::Node->new_from_uri( $turl );
			my $tst		= RDF::Redland::Statement->new( $image, $p_type, $t_image );
			unless ($model->contains_statement( $tst )) {
				$model->add_statement( $tst );
			}
			my $st		= RDF::Redland::Statement->new( $image, $p_thumb, $thumb );
			unless ($model->contains_statement( $st )) {
				$model->add_statement( $st );
			}
		}
		if (my $title = delete $entry->{'column_values'}{'image_title'}) {
			my $node		= $entry->get_rdf_node;
			my ($l_title)	= RDF::Redland::Node->new_literal( $title );
			my $st		= RDF::Redland::Statement->new( $image, $p_title, $l_title );
			unless ($model->contains_statement( $st )) {
				$model->add_statement( $st );
			}
		}
	}
});

sub _get_thumbnail {
	my $ctx		= shift;
	my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
	my $model	= $entry->driver->model;
	my $t_image	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/Image' );
	my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topic_of_type($t_image);
	if ($topic) {
		my $p_thumb	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/thumbnail' );
		my ($thumb)	= $model->targets( $topic, $p_thumb );
		if ($thumb and $thumb->is_resource) {
			return $thumb;
		} elsif ($topic and $topic->is_resource) {
			return $topic;
		}
	}
}

MT::Template::Context->add_tag( EntryTopicImageThumbnail => sub {
	my $warnings;
	local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
	my $ctx		= shift;
	my $args	= shift;
	my $thumb	= _get_thumbnail( $ctx );
	if ($thumb and $thumb->is_resource) {
		my $url	= $thumb->uri->as_string;
		return $url;
	} else {
		return "\n<!--\n$warnings\n-->\n" if ($debug);
		return '';
	}
} );

MT::Template::Context->add_tag( EntryTopicImageThumbnailWidth => sub {
	my $warnings;
	local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
	my $ctx		= shift;
	my $args	= shift;
	my $t_image	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/Image' );
	my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
	my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topic_of_type($t_image);
	my $thumb	= _get_thumbnail( $ctx );
	if ($thumb and $thumb->equals( $topic ) and exists $args->{default}) {
		return $args->{default};
	} elsif ($thumb and $thumb->is_resource) {
		my $url	= $thumb->uri->as_string;
		my ($w, undef)	= _get_image_dimensions( $ctx, $url );
		return $w;
	} else {
		return "\n<!--\n$warnings\n-->\n" if ($debug);
		return '';
	}
} );

MT::Template::Context->add_tag( EntryTopicImageThumbnailHeight => sub {
	my $warnings;
	local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
	my $ctx		= shift;
	my $args	= shift;
	my $t_image	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/Image' );
	my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
	my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topic_of_type($t_image);
	my $thumb	= _get_thumbnail( $ctx );
	if ($thumb and $thumb->equals( $topic ) and exists $args->{default}) {
		return $args->{default};
	} elsif ($thumb and $thumb->is_resource) {
		my $url	= $thumb->uri->as_string;
		my (undef, $h)	= _get_image_dimensions( $ctx, $url );
		return $h;
	} else {
		return "\n<!--\n$warnings\n-->\n" if ($debug);
		return '';
	}
} );

MT::Template::Context->add_tag( EntryTopicImageWidth => sub {
	my $warnings;
	local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
	my $ctx		= shift;
	my $args	= shift;
	my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
	my $model	= $entry->driver->model;
	my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topics();
	if ($topic and $topic->is_resource) {
		my $url	= $topic->uri->as_string;
		my ($w, undef)	= _get_image_dimensions( $ctx, $url );
		return $w;
	} else {
		return "\n<!--\n$warnings\n-->\n" if ($debug);
		return '';
	}
} );

MT::Template::Context->add_tag( EntryTopicImageHeight => sub {
	my $warnings;
	local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
	my $ctx		= shift;
	my $args	= shift;
	my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
	my $model	= $entry->driver->model;
	my ($topic)	= $ctx->stash('rdf_topic') || $entry->rdf_topics();
	if ($topic and $topic->is_resource) {
		my $url	= $topic->uri->as_string;
		my (undef, $h)	= _get_image_dimensions( $ctx, $url );
		return $h;
	} else {
		return "\n<!--\n$warnings\n-->\n" if ($debug);
		return '';
	}
} );

sub _get_image_dimensions {
	my $ctx		= shift;
	my $url		= shift;
	
	my $node		= RDF::Redland::Node->new_from_uri( $url );
	my $p_width		= RDF::Redland::Node->new_from_uri( 'http://www.w3.org/2003/12/exif/ns#width' );
	my $p_height	= RDF::Redland::Node->new_from_uri( 'http://www.w3.org/2003/12/exif/ns#height' );
	my $entry		= $ctx->stash('entry') || $ctx->stash('entry_preview');
	my $model		= $entry->driver->model;
	my $w			= MT::Plugins::Redland::CMS::Topic::get_first_literal($model, $node, $p_width) || $ctx->stash( "width:$url" );
	my $h			= MT::Plugins::Redland::CMS::Topic::get_first_literal($model, $node, $p_height) || $ctx->stash( "height:$url" );
	if ($w and $h) {
		return ($w, $h);
	} else {
		require File::Temp;
		require Image::Info;
		require LWP::Simple;
		my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
		my $model	= $entry->driver->model;
		my ($fh, $filename) = File::Temp::tempfile();
		close($fh);
		LWP::Simple::getstore($url, $filename);
		my $info	= Image::Info::image_info($filename);
		($w, $h)	= Image::Info::dim( $info );
		unlink($filename);
		if ($w and $h) {
			my $l_width		= RDF::Redland::Node->new_literal( $w );
			my $l_height	= RDF::Redland::Node->new_literal( $h );
			$model->add_statement(RDF::Redland::Statement->new( $node, $p_width, $l_width ));
			$model->add_statement(RDF::Redland::Statement->new( $node, $p_height, $l_height ));
			$ctx->stash( "width:$url", $w );
			$ctx->stash( "height:$url", $h );
			return ($w, $h);
		}
	}
}

######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms_topic_image.pl,v $
 Revision 1.6  2005/04/11 05:39:23  greg
 - added image thumbnail url and image title fields
 - added image template tags: EntryTopicImageWidth, EntryTopicImageHeight
 - added thumbnail template tags: EntryTopicImageThumbnail, EntryTopicImageThumbnailWidth, EntryTopicImageThumbnailHeight

 Revision 1.5  2005/04/06 15:09:45  greg
 - bumped version number to 0.005
 - updated email address in AUTHOR POD section

 Revision 1.4  2005/04/05 05:36:19  greg
 - during update, now uses the remove_rdf_topics_of_type method from redland_cms_topic.pl

 Revision 1.3  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.2  2005/04/04 07:46:37  greg
 - moved topic_of_type() and topics() into MT::Entry namespace as
   rdf_topic_of_type() and rdf_topics()

 Revision 1.1  2005/04/03 01:43:30  greg
 - added redland_cms_topic_image.pl for foaf:Image topics
 - added stub plugin for Redland Template Tags


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <greg@evilfunhouse.com>

=cut
